#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-amga-server-config v. 1.0.0
#
# Post-installation script for configuring the gLite AMGA client
# Robert Harakaly  < robert.harakaly@cern.ch >
#
# Version info: $Id: glite-amga-client-config.py,v 1.6.2.1 2006/01/18 11:06:13 harakaly Exp $
#
# Usage: python glite-amga-client-config [-c|-v|-h|--start|--stop|--status|--configure--help]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --start     start the service
#        --stop      start the service
#        --status    get the service status (o=running, 1=idle) 
#        --configure configure the service and database (default)
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################
from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams
#from gliteRgmaServicetool import gliteRgmaServicetool
import os
import getopt
import sys

class glite_amga_client:
    def __init__(self, params):
        self.params = params
        self.verbose = 0
        self.version = "1.0.0"
        self.name = "glite-amga-client"
        self.friendly_name = "gLite AMGA client"
        params['module.version'] = self.version
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure the service'
        print '    --siteconfig=<URL>  use site configuration file at URL'
        print '\n'


    def configure(self):
        glib.printInfoMessage("Creating mdserver.config file")
        conf = open ("%s/etc/mdclient.config" % params['GLITE_LOCATION'],"w")
        for param in self.params.keys():
            if param.startswith("amga.client"):
                if param == 'amga.client.securityOptions':
                    for option in params[param]:
                        conf.write("%s = 1\n" % option)
                else:
                    if not params[param] == '':
                        conf.write("%s = %s\n" % (param.split(".")[2], params[param]))
        conf.close()
        return 0
        
    def start(self):
        return 0
    def stop(self):
        return 0
    def status(self):
        return 0

def setDefaults():
    params['amga.client.Port'] = 8822
    params['amga.client.PermissionMask'] = 'rw-'
    params['amga.client.GroupMask'] = 'r--'
    params['amga.client.Home'] = '/'
    params["amga.client.CertFile"] = params['host.certificate.file']
    params['amga.client.KeyFile'] = params['host.key.file']
    params['amga.client.TrustedCertDir'] = params['ca.certificates.dir']
    
def set_env():
    glib.export('GLITE_LOCATION')
    glib.export('GLITE_LOCATION_VAR')
    glib.export('GLITE_LOCATION_LOG')
    glib.export('GLITE_LOCATION_TMP')
    glib.export('GLOBUS_LOCATION')
    glib.export('GPT_LOCATION')
    glib.export('LD_LIBRARY_PATH')
    glib.export('JAVA_HOME')
    
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'], 0777)
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    glib.addEnvPath("PATH", "%s/bin" % os.environ["JAVA_HOME"])
    glib.addEnvPath("PATH", "%s/externals/bin" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("PATH", "%s/bin" % os.environ["GLITE_LOCATION"])

    glib.addEnvPath("LD_LIBRARY_PATH", "/usr/local/lib")
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ['GLOBUS_LOCATION'])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/externals/lib" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ["GLITE_LOCATION"])

    glib.addEnvPath('PYTHONPATH',"%s/lib/python2.2/site-packages/" % os.environ['GLITE_LOCATION'])
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid() == 0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    
    # Load parameters
    params = ConfigParams()
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-amga-client.cfg.xml" % glib.getScriptPath()], params):
        print "An error occurred while configuring AMGA client"
        sys.exit(1)
    
    setDefaults()
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    gLib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    # Instantiate the service classes
    service = glite_amga_client(params)
    service.verbose = verbose

    # Instantiate the rgma servicetool class
#    if not params.has_key('rgma.servicetool.activate'):
#        params['rgma.servicetool.activate'] = 'true'
#    if params['rgma.servicetool.activate'] == 'true':
#        rgmaServicetool = gliteRgmaServicetool()
#        rgmaServicetool.verbose = verbose

    # Command line opts if any
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', \
            'configure', 'start', 'stop', 'status', 'siteconfig=', 'subservice'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:

        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        if o == "--configure":
            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)
            
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)
                            
            # Configure the service
            return_value = service.configure()
            
            if return_value == 0:
                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                sys.exit(0)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--restart":
            sys.exit(0)
            
        if o == "--start":
            sys.exit(0)

        if o == "--stop":
            sys.exit(0)

        if o == "--status":
            sys.exit(service.status())

        if o == "--subservice":
            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)

            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)

            # Configure the service
            return_value = service.configure()

            if return_value == 0:
                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                sys.exit(0)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
            